//+------------------------------------------------------------------+
//|                                               Moving Average.mq4 |
//|                      Copyright  2005, MetaQuotes Software Corp. |
//|                                       http://www.metaquotes.net/ |
//+------------------------------------------------------------------+
#define MAGICMA  20050610

extern double Lots               = 0.1;
extern double MaximumRisk        = 0.02;
extern double DecreaseFactor     = 3;
extern double MovingPeriod       = 12;
extern double MovingShift        = 6;
//+------------------------------------------------------------------+
//| Calculate open positions                                         |
//+------------------------------------------------------------------+
int CalculateCurrentOrders(string symbol)
  {
   int buys=0,sells=0;
//----
   for(int i=0;i<OrdersTotal();i++)
     {
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==false) break;
      if(OrderSymbol()==Symbol() && OrderMagicNumber()==MAGICMA)
        {
         if(OrderType()==OP_BUY)  buys++;
         if(OrderType()==OP_SELL) sells++;
        }
     }
//---- return orders volume
   if(buys>0) return(buys);
   else       return(-sells);
  }
//+------------------------------------------------------------------+
//| Calculate optimal lot size                                       |
//+------------------------------------------------------------------+
double LotsOptimized()
  {
   double lot=Lots;
   int    orders=HistoryTotal();     // history orders total
   int    losses=0;                  // number of losses orders without a break
//---- select lot size
   lot=NormalizeDouble(AccountFreeMargin()*MaximumRisk/1000.0,1);
//---- calcuulate number of losses orders without a break
   if(DecreaseFactor>0)
     {
      for(int i=orders-1;i>=0;i--)
        {
         if(OrderSelect(i,SELECT_BY_POS,MODE_HISTORY)==false) { Print("Error in history!"); break; }
         if(OrderSymbol()!=Symbol() || OrderType()>OP_SELL) continue;
         //----
         if(OrderProfit()>0) break;
         if(OrderProfit()<0) losses++;
        }
      if(losses>1) lot=NormalizeDouble(lot-lot*losses/DecreaseFactor,1);
     }
//---- return lot size
   if(lot<0.1) lot=0.1;
   return(lot);
  }
//+------------------------------------------------------------------+
//| Check for open order conditions                                  |
//+------------------------------------------------------------------+
void CheckForOpen()
  {
   double ma;
   int    res;
//---- go trading only for first tiks of new bar
   if(Volume[0]>1) return;
//---- get Moving Average 
   ma=iMA(NULL,0,MovingPeriod,MovingShift,MODE_SMA,PRICE_CLOSE,0);
//---- sell conditions
   if(Open[1]>ma && Close[1]<ma)  
     {
      res=OrderSend(Symbol(),OP_SELL,LotsOptimized(),Bid,3,0,0,"",MAGICMA,0,Red);
      return;
     }
//---- buy conditions
   if(Open[1]<ma && Close[1]>ma)  
     {
      res=OrderSend(Symbol(),OP_BUY,LotsOptimized(),Ask,3,0,0,"",MAGICMA,0,Blue);
      return;
     }
//----
  }
//+------------------------------------------------------------------+
//| Check for close order conditions                                 |
//+------------------------------------------------------------------+
void CheckForClose()
  {
   double ma;
//---- go trading only for first tiks of new bar
   if(Volume[0]>1) return;
//---- get Moving Average 
   ma=iMA(NULL,0,MovingPeriod,MovingShift,MODE_SMA,PRICE_CLOSE,0);
//----
   for(int i=0;i<OrdersTotal();i++)
     {
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==false)        break;
      if(OrderMagicNumber()!=MAGICMA || OrderSymbol()!=Symbol()) continue;
      //---- check order type 
      if(OrderType()==OP_BUY)
        {
         if(Open[1]>ma && Close[1]<ma) OrderClose(OrderTicket(),OrderLots(),Bid,3,White);
         break;
        }
      if(OrderType()==OP_SELL)
        {
         if(Open[1]<ma && Close[1]>ma) OrderClose(OrderTicket(),OrderLots(),Ask,3,White);
         break;
        }
     }
//----
  }
//+------------------------------------------------------------------+
//| Start function                                                   |
//+------------------------------------------------------------------+
void start()
  {
//---- check for history and trading
   if(Bars<100 || IsTradeAllowed()==false) return;
//---- calculate open orders by current symbol
   if(CalculateCurrentOrders(Symbol())==0) CheckForOpen();
   else                                    CheckForClose();
//----
  }
  
//+------------------------------------------------------------------+
//| Start function                                                   |
//+------------------------------------------------------------------+
void deinit()
  {
    
    //---     []_[]_[].csv   terminal_dir\experts\files                                                                    
    string strFileName;
    string Prefix =WindowExpertName() +  "_";
    int Prefix_length =  StringLen(Prefix);
        //---   .         (  )
    datetime dtCurrentTime=GlobalVariableSet("tmpVar",777) + 14400; // +14400 Seconds = 4 Hour - ,    
                                                                         // +10800 Seconds = 3 Hour - ,    
    strFileName=SetFileName(Prefix, dtCurrentTime, Prefix_length);            
  
  
  int h;                      //        
  
  //---    ,     
  h=FileOpen(strFileName,FILE_CSV|FILE_WRITE,';');
  

//--- "_"     ,  MFE_MAE  MVtE  ,    
   if ( ! IsOptimization() )  //&& GlobalVariableGet("AllowWriteReportCSV") > 0 
    {
    int tmr = GetTickCount();
    Print("  csv   MFE_MAE  MVtE .....");
    double thisOrderMaxProfit,thisOrderMinProfit,thisOrderProfit,thisOrderProfitSwap,BalCurve;
    /* //      CSV- 
    string PrefixMFE =WindowExpertName() + "_MFE_MAE_";
    int Prefix_lengthMFE = StringLen(PrefixMFE);
    dtCurrentRecordTime=GlobalVariableSet("SetFileNameKloss",995) + TimeCorrection; // +14400 Seconds = 4 Hour -    
    string strFNameMFE=SetFileNameKloss(PrefixMFE, dtCurrentRecordTime, Prefix_lengthMFE);//           Kloss          
    int hMFE=FileOpen(strFNameMFE,FILE_CSV|FILE_READ|FILE_WRITE,';');*/
    FileWrite(h   ,"_","()", "", "=======","=======","=======","=======","=======","=======","=======","=======","=======","======="); 
    FileWrite(h   ," ","Ticket"   ,"",                 "",          "OrderLot",               "Profit/Ticket",                 "Swap/Ticket",
                   "OrderOpenPrice",                     "OrderClosePrice",                    "OrderOpenTime",                                "OrderCloseTime",
                   "MFE",                          "MAE",                              "MVtE",
                   "","1-", //    , ..  VBA    ""
                   "",   "1440",   "", "120",     "",       "365");
    //         ,   ,       = 2 
    FileWrite(h   ,"");              
             
    int ix = 0; //    
    for (int i=0; i<OrdersHistoryTotal(); i++) 
      {
      OrderSelect(i,SELECT_BY_POS,MODE_HISTORY);
      
      if(OrderSymbol() != Symbol()) continue;
      //if(GetMagicRoot(OrderMagicNumber()) != GetMagicRoot(nMagic)) continue;
      if ( !(OrderType()==OP_BUY || OrderType()==OP_SELL))  continue;
      ix++;
      GetProfit(OrderTicket(),thisOrderMaxProfit,thisOrderMinProfit); //        
      thisOrderProfit =OrderProfit();
      thisOrderProfitSwap =OrderProfit()+ OrderSwap();
      BalCurve += thisOrderProfitSwap;
      //FileWrite(hMFE,ix,OrderTicket(),DoubleToStr(thisOrderMaxProfit,2),DoubleToStr(thisOrderMinProfit,2), DoubleToStr((thisOrderMaxProfit + thisOrderMinProfit)/2,2), DoubleToStr(thisOrderProfit,2),
      //               DoubleToStr(thisOrderProfitSwap - thisOrderProfit,2),TimeToStr(OrderOpenTime()),TimeToStr(OrderCloseTime()), DoubleToStr(BalCurve,2),DoubleToStr(TableTSSF[ix][4],6));
      FileWrite(h   ,ix,OrderTicket(),GetNameOP(OrderType()),OrderMagicNumber(),DoubleToStr(OrderLots(),2),DoubleToStr(thisOrderProfit,2),DoubleToStr(thisOrderProfitSwap - thisOrderProfit,2),
                    DoubleToStr(OrderOpenPrice(),Digits),DoubleToStr(OrderClosePrice(),Digits),TimeToStr(OrderOpenTime(),TIME_DATE|TIME_SECONDS),TimeToStr(OrderCloseTime(),TIME_DATE|TIME_SECONDS),
                    DoubleToStr(thisOrderMaxProfit,2),DoubleToStr(thisOrderMinProfit,2), DoubleToStr((thisOrderMaxProfit + thisOrderMinProfit)/2,2),
                    DoubleToStr(BalCurve,2),0,  //TablePosInfo[ix-1][2],TablePosInfo[ix-1][3]
                    0,0,0,0,0,0);
      }
    //          ,   ,       .
    //    (  )      [_]    [End_Report]
    FileWrite(h              ,"");//  _QoSUni       Excel
    //if (ix != ArrayRange(TableTSSF,0)-1 ) FileWrite(h   ,"   MAE  TSSF");
    //if (ix != RecIndexTablePosInfo() ) FileWrite(h   ,"   MAE  TablePosInfo");
    //Print ("ix= ",ix,"   TableTSSF= ",ArrayRange(TableTSSF,0));
    //FileClose(hMFE); 
    FileWrite(h           ,""); 
    Print("  csv   MFE_MAE  MVtE .   (): ", GetTickCount() - tmr);
    }
  //---    MFE_MAE
  //==================================================================================


//----
  }

//+------------------------------------------------------------------+
//| function >> SetFileName        []_[]_[].csv   terminal_dir\experts\files                    |
//++-----------------------------------------------------------------+
string SetFileName(string Prefix, datetime dt, int Prefix_length)
  {
  
  string RESULT=Prefix;
  
  if (IsOptimization()) RESULT=StringConcatenate(RESULT,"OPT_"); 
  
  if (TimeDay(dt)>9) RESULT=StringConcatenate(RESULT,TimeDay(dt)); else RESULT=StringConcatenate(RESULT,"0",TimeDay(dt));
  if (TimeMonth(dt)>9) RESULT=StringConcatenate(RESULT,TimeMonth(dt)); else RESULT=StringConcatenate(RESULT,"0",TimeMonth(dt));
  RESULT=StringConcatenate(RESULT,StringSubstr(DoubleToStr(TimeYear(dt),0),2));
  RESULT=StringConcatenate(RESULT,"_");
  if (TimeHour(dt)>9) RESULT=StringConcatenate(RESULT,TimeHour(dt)); else RESULT=StringConcatenate(RESULT,"0",TimeHour(dt));
  if (TimeMinute(dt)>9) RESULT=StringConcatenate(RESULT,TimeMinute(dt)); else RESULT=StringConcatenate(RESULT,"0",TimeMinute(dt));
  RESULT=StringConcatenate(RESULT,".csv");
    // =StringConcatenate("VtTestSignal_",TiTimeMonth(dt)meDay(dtCurrentTime),TimeMonth(dtCurrentTime),TimeYear(dtCurrentTime),
    //                    "_",TimeHour(dtCurrentTime),TimeMinute(dtCurrentTime),".csv");
  
  //---  .    Prefix    = 777
  int    var_total=GlobalVariablesTotal();
         for (int i=0;i<var_total;i++)
          {
           if (StringSubstr( GlobalVariableName(i), 0, Prefix_length)==Prefix) GlobalVariableDel(GlobalVariableName(i));
           if (GlobalVariableGet( GlobalVariableName(i)) == 777 ) GlobalVariableDel(GlobalVariableName(i));       
          }  
  //---  .         , . 777 
  GlobalVariableSet(RESULT,777);   
  return(RESULT);
  }
  
//+------------------------------------------------------------------+
//|             |
//|      MAE_MFE_DrawDowns.mqh     |
//++-----------------------------------------------------------------+
bool GetProfit(int ticket,double& maxProf,double& minProf )
   {
   bool res=true;
//----
   string symbol,crossPair,firstCurrency,secondCurrency,baseCurrency=AccountCurrency();
   int barOpen,barClose,type,maxPoint,minPoint,err;
   double openPrice, closePrice, maxPrice, minPrice, spread, point, pointCost, lots, price;
   datetime foundedTime;
   minProf=0;
   maxProf=0;
Print("     #",ticket); //
   if (ticket<0) return(res);
   if (OrderSelect(ticket,SELECT_BY_TICKET))
      {
      symbol=OrderSymbol();
      firstCurrency=StringSubstr(OrderSymbol(),0,3);
      secondCurrency=StringSubstr(OrderSymbol(),3,3);//
    Print("base=",baseCurrency,"   firstCurrency=",firstCurrency,"   secondCurrency=",secondCurrency);//
      type=OrderType();
      openPrice=OrderOpenPrice();
      closePrice=OrderClosePrice();
      barOpen=iBarShift(symbol,Period(),OrderOpenTime(),true);
      barClose=iBarShift(symbol,Period(),OrderCloseTime(),true);  //PERIOD_M1
   Print (" Open  ", barOpen , "  close ",barClose);
      //foundedTime=iTime(symbol,Period(),bar);   //PERIOD_M1
      spread=MarketInfo(symbol,MODE_SPREAD);
      point=MarketInfo(symbol,MODE_POINT);
      lots=OrderLots();
    //---   1    .  
      if (firstCurrency==baseCurrency)
         {
         price=(openPrice + closePrice)/2;   //  
         Print("price=",price);//
         pointCost=point*MarketInfo(symbol,MODE_LOTSIZE)/price;
         }
      if (secondCurrency==baseCurrency)
         {
         pointCost=point*MarketInfo(symbol,MODE_LOTSIZE);
         }
      if (firstCurrency!=baseCurrency && secondCurrency!=baseCurrency)
         {
         /*Print("    - ",symbol, " .     !");
         if (MarketInfo(secondCurrency+baseCurrency,MODE_BID)>0) 
            {
            crossPair=StringConcatenate(secondCurrency,baseCurrency);
            price=(iHigh(crossPair,Period(),bar)+iLow(crossPair,Period(),bar))/2;    //PERIOD_M1
            pointCost=point*MarketInfo(symbol,MODE_LOTSIZE)*price;
            }
         if (MarketInfo(baseCurrency+secondCurrency,MODE_BID)>0) 
            {
            crossPair=StringConcatenate(baseCurrency,secondCurrency);
            price=(iHigh(crossPair,Period(),bar)+iLow(crossPair,Period(),bar))/2;    //PERIOD_M1
            Print(crossPair,"=",price,"   ",TimeToStr(foundedTime));//
            pointCost=point*MarketInfo(symbol,MODE_LOTSIZE)/price;
            }

         err=GetLastError();
         Print("    ",crossPair," M1  ",TimeToStr(at_minute)); */
         }   
      Print("    #",ticket,"  ",pointCost); //
    //-----------------------------------
    
         
      /*if (foundedTime!=TimeRoundeMinute(at_minute)) 
         {
         //Print("    ",symbol," M1  ",TimeToStr(at_minute)); //
         //Print("        foundedTime=",TimeToStr(foundedTime)); //
        }  */ 
      if (type==OP_BUY)
         {
         maxProf=(iHigh(symbol,0,iHighest(symbol,0,MODE_HIGH,barOpen-barClose+1,barClose))-openPrice)/point;   //PERIOD_M1
         minProf=(iLow(symbol,0,iLowest(symbol,0,MODE_LOW,barOpen-barClose+1,barClose))-openPrice)/point;     //PERIOD_M1
          //Print("Buy  Hi=",iHigh(symbol,0,iHighest(symbol,0,MODE_HIGH,barOpen-barClose+1,barClose)),
          //      "   Low=",iLow(symbol,0,iLowest(symbol,0,MODE_LOW,barOpen-barClose+1,barClose))," at ");
         }
      if (type==OP_SELL)
         {
         maxProf=(openPrice-iLow(symbol,0,iLowest(symbol,0,MODE_LOW,barOpen-barClose+1,barClose)))/point - spread;     //PERIOD_M1
         minProf=(openPrice-iHigh(symbol,0,iHighest(symbol,0,MODE_HIGH,barOpen-barClose+1,barClose)))/point - spread;       //PERIOD_M1
          //Print("Sell  Hi(max)=",iLow(symbol,0,iLowest(symbol,0,MODE_LOW,barOpen-barClose+1,barClose)),"   Low(min)=",
          //      iHigh(symbol,0,iHighest(symbol,0,MODE_HIGH,barOpen-barClose,barClose+1))," at ");         
         }
     //Print("maxProf=",maxProf,"   minProf=",minProf," at ");  // 
   
      maxProf=maxProf*lots*pointCost;
      minProf=minProf*lots*pointCost;
     Print("maxProf=",maxProf,"   minProf=",minProf," at ");  //   
      }
   else
      {
      res=false;
      Print("   #",ticket," .  GetProfit()");
      }     
//----
   return(res);   
   }
   
//+----------------------------------------------------------------------------+
//|      :   . aka KimIV,  http://www.kimiv.ru                   |
//+----------------------------------------------------------------------------+
//|     : 01.09.2005          >> GetNameOP                               |
//|   :                          |
//+----------------------------------------------------------------------------+
//|  :                                                                |
//|    op -                                       |
//++-kim-----------------------------------------------------------------------+
string GetNameOP(int op) {
  switch (op) {
    case OP_BUY      : return("Buy");
    case OP_SELL     : return("Sell");
    case OP_BUYLIMIT : return("Buy Limit");
    case OP_SELLLIMIT: return("Sell Limit");
    case OP_BUYSTOP  : return("Buy Stop");
    case OP_SELLSTOP : return("Sell Stop");
    default          : return("Unknown Operation");
  }
}    
//+------------------------------------------------------------------+